import * as Express from 'express';

import {Article_type, Article, Article_} from './Article';

const PLM = Express();
PLM.use(Express.json()); // For POST (and PUT) requests (based on 'body-parser' library in Node.js)...
const port = process.env['PORT'] || 1963; // 'process' => "@types/node"
const server = PLM.listen(port);
console.log(`Server ready to accept requests on port ${port}... ctrl + C to abort...`);

const Articles = new Array<Article>();
const {error, value} = Article_.validate({
    reference: 'CD100',
    designation: 'camion demenagement bleu',
    type_fabrication_achat: 'fab. par lot',
    unite_achat_stock: 'unite',
    delai_en_semaine: 2,
    lot_de_reapprovisionnement: 200,
    stock_maxi: 600,
    PF_ou_MP_ou_Piece_ou_SE: Article_type.PF
});
if (error === undefined) {
    console.log("Article validé (succès) : " + value.reference);
    Articles.push({
        reference: 'CD100',
        designation: 'Camion demenagement bleu',
        type_fabrication_achat: 'Fab. par lot',
        unite_achat_stock: 'unite',
        delai_en_semaine: 2,
        lot_de_reapprovisionnement: 200,
        stock_maxi: 600,
        PF_ou_MP_ou_Piece_ou_SE: Article_type.PF
    });
} else
    console.log("Article invalidé (échec) : " + error.message);

/* Utilisation de 'curl' sous macOS -> 'curl --help' */
/* Utilisation de 'curl' sous Windows 11 -> 'curl -?' */
PLM.get('/', (request, response) => { // Test: 'curl http://localhost:1963/'
    response.send('<h1 style="color: green;">"PLM.Node.js.ts": Restful Web services</h1>');
});
PLM.get('/api/PLM/Articles', (request, response) => { // Test: 'curl http://localhost:1963/api/PLM/Articles'
    response.send(Articles);
});
PLM.get('/api/PLM/Articles/:reference', (request, response) => { // Test: 'curl http://localhost:1963/api/PLM/Articles/CD100'
    const article = Articles.find((a: Article) => a.reference === request.params.reference);
    if (!article) response.status(404).send('<h1 style="color: red;">Not found: ' + request.params.reference + '</h1>');
    response.send(article);
});
// Test: curl -X POST http://localhost:1963/api/PLM/Nouvel_article -H 'Content-Type: application/json' --data '{"reference":"CC201","designation":"Camion citerne rouge","type_fabrication_achat":"Fab. a la commande","unite_achat_stock":"unite","delai_en_semaine":2,"lot_de_reapprovisionnement":150,"stock_maxi":600,"PF_ou_MP_ou_Piece_ou_SE":"PF"}'
PLM.post("/api/PLM/Nouvel_article", (request, response) => {
    /* Création article (Windows 11) :
    $CC201 = @{
        reference = 'CC201'
        designation = 'Camion citerne rouge'
        type_fabrication_achat = 'Fab. a la commande'
        unite_achat_stock = 'unite'
        delai_en_semaine = 2
        lot_de_reapprovisionnement = 150
        stock_maxi = 600
        PF_ou_MP_ou_Piece_ou_SE = 'PF'
    }|ConvertTo-Json
    echo $CC201
    */
    /* (Windows 11) :
    $Nouvel_article = @{
        Body        = $CC201
        ContentType = 'application/json'
        Method      = 'POST'
        Uri         = 'http://localhost:1963/api/PLM/Nouvel_article'
    }
    */
    /* (Windows 11) :
    Invoke-RestMethod @Nouvel_article
    */
    // console.log(JSON.stringify(request.body));
    const article: Article = request.body;
    const {error, value} = Article_.validate({
        reference: article.reference, // Unicity required as well...
        designation: article.designation, // Unicity required as well...
        type_fabrication_achat: article.type_fabrication_achat,
        unite_achat_stock: article.unite_achat_stock,
        delai_en_semaine: article.delai_en_semaine,
        lot_de_reapprovisionnement: article.lot_de_reapprovisionnement,
        stock_maxi: article.stock_maxi,
        PF_ou_MP_ou_Piece_ou_SE: article.PF_ou_MP_ou_Piece_ou_SE
    });
    if (error === undefined) {
        response.status(201).json({
            message: "Article validé (succès) : " + value.reference
        });
        Articles.push(article);
    } else {
        response.status(400).json({
            message: "Article invalidé (échec) : " + error.message
        });
    }
});
